/*
 * Oppussingsprosjekt.java   E.L. 2004-02-21
 *
 * Klassen vedlikeholder et register over flater og et register over materialtyper.
 * En videreutvikling av klassen med samme navn fra kapittel 9 (programliste 9.4).
 *
 */
package Oppussingsprosjekt;

import java.util.ArrayList;
public class Oppussingsprosjekt {
  private String navn;
  private ArrayList<Flate> alleFlater = new ArrayList<Flate>();
  private ArrayList<Materiale> alleMaterialtyper = new ArrayList<Materiale>();

  public Oppussingsprosjekt(String startNavn) {
    navn = startNavn;
  }

  public String finnNavn() {
    return navn;
  }

  /*
   * Metoden registrerer en ny materialtype. Dersom en materialtype med dette
   * navnet er registrert fra fr, returneres denne materialtypen,
   * ellers returneres den nye materialtypen.
   */
  public Materiale registrerNyttMateriale(Materiale nyMaterialtype) {
    Materiale denne = finnMateriale(nyMaterialtype.finnNavn());
    if (denne == null) {
      alleMaterialtyper.add(nyMaterialtype);
      return nyMaterialtype;
    }
    else return denne;
  }

  /*
   * Metoden sker etter en materialtype med et gitt navn.
   * Dersom det ikke finnes, returneres null.
   */
  public Materiale finnMateriale(String materialenavn) {
    for (Materiale dette : alleMaterialtyper) {
      if ((dette.finnNavn()).equals(materialenavn)) return dette;
    }
    return null;
  }

  /* Metoden er bygd opp p samme mte som registrerNyttMateriale() */
  public Flate registrerNyFlate(Flate nyFlate) {
    Flate denne = finnFlate(nyFlate.finnNavn());
    if (denne == null) {
      alleFlater.add(nyFlate);
      /*
       * Materialtype kan vre knyttet til denne flaten.
       * Hvis denne materialtypen ikke er registrert her fra fr, registrerer vi den n.
       */
      Materiale materiale = nyFlate.finnMateriale();
      if (materiale != null) registrerNyttMateriale(materiale);
      return nyFlate;
    }
    else return denne;  // flate med dette navnet er registrert fra fr
  }

  /* Metoden er bygd opp p samme mte som finnMaling() */
  public Flate finnFlate(String Flatenavn) {
    for (Flate denne : alleFlater) {
      if ((denne.finnNavn()).equals(Flatenavn)) return denne;
    }
    return null;
  }

  /* Finner prosjektets totalpris */
  public double finnTotalpris() {
    double totalpris = 0.0;
    for (Flate flaten : alleFlater) {
      Materiale materiale = flaten.finnMateriale();
      if (materiale != null) totalpris += materiale.finnTotalpris(flaten);
    }
    return totalpris;
  }

  /*
   * Metoder som kan brukes ved sekvensiell sk gjennom
   *  alle materialeer eller alle flater
   */

  public int finnAntMaterialer() {
    return alleMaterialtyper.size();
  }

  public Materiale finnMateriale(int indeks){
    if (indeks >= 0 && indeks < alleMaterialtyper.size()) {
      return alleMaterialtyper.get(indeks);
    }
    else return null;
  }

  public int finnAntFlater() {
    return alleFlater.size();
  }

  public Flate finnFlate(int indeks){
    if (indeks >= 0 && indeks < alleFlater.size()) {
      return alleFlater.get(indeks);
    }
    else return null;
  }

  public String lagPrisOgBehovDetaljer() {
    java.text.DecimalFormat utskriftFormat = new java.text.DecimalFormat("####0.00");
    String resultat = "";
    double totalpris = 0.0;
    for (Flate flaten : alleFlater) {
      resultat += flaten.toString();
      Materiale materiale = flaten.finnMateriale();
      if (materiale != null) {
        double behov = materiale.finnMaterialbehov(flaten);
        double pris = materiale.finnTotalpris(flaten);
        resultat += ", behov: " + utskriftFormat.format(behov) +
                    " liter, pris " + utskriftFormat.format(pris) + " kr.\n";
        totalpris += pris;
      } else resultat += " Materialbehov ikke bestemt\n";
    }
    resultat += "Totalpris " + utskriftFormat.format(totalpris) + " kr.";
    return resultat;
  }

  public String toString() {
    String resultat = "Alle flater:\n";
    for (Flate enFlate : alleFlater) resultat += enFlate + "\n";

    resultat += "\nAlle materialtyper:\n";
    for (Materiale etMat : alleMaterialtyper) resultat += etMat + "\n";
    return resultat;
  }
}